==> entodicton-5.51.106.tgz: package/src/generators.js <==
const gs = (g) => (contexts, separator, last_separator) => {
  s = ''
  if (!separator) {
    separator = ' '
  }
  if (!last_separator) {
    last_separator = separator
  }
  next_separator = ''
  for (let i = 0; i < contexts.length; ++i) {
    const context = contexts[i]
    value = g(context)
    if (i > 0) {
      if (i == contexts.length-1) {
        next_separator = last_separator
      } else {
        next_separator = separator
      }
    }
    s += next_separator + value
  }
  return s
}

class Generator {
  constructor(matcher, action) {
    this.matcher = matcher;
    this.action = action;
  }

  toString() {
    return `Generator(${this.matcher}, ${this.action})`
  }

  matches(global, context) {
    try {
      return this.matcher({global, context})
    } catch ( e ) { 
      // don't care
    }
  }

  apply(global, g, context) {
    return this.action({global, g, context, gs: gs(g)})
  }
}

class Generators {
  constructor(generators) {
    this.generators = generators
    this.logs = []
  };

  apply(global, contexts) {
    if (Array.isArray(contexts)) {
    } else if ( typeof contexts == 'object' ) {
      contexts = [contexts]
    } else {
      return String(contexts)
    }
    
    const contexts_prime = [];
    for (const icontext in contexts) {
      const context = contexts[icontext]
      let context_prime = JSON.stringify(context);
      for (let igenerator = 0; igenerator < this.generators.length; ++igenerator) {
        const generator = this.generators[igenerator];
        if (generator.matches(global, context)) {
          const g = (context) => this.apply(global, context)
          this.logs.push(`Generators: applied ${generator.toString()}\n to\n ${JSON.stringify(context)}`)
          context_prime = generator.apply(global, g, context);
          break
        }
      }
      contexts_prime.push(context_prime);
    }
    return contexts_prime;
  }
}

module.exports = { Generator, Generators };

==> entodicton-5.51.116.tgz: package/src/generators.js <==
const gs = (g) => (contexts, separator, last_separator) => {
  s = ''
  if (!separator) {
    separator = ' '
  }
  if (!last_separator) {
    last_separator = separator
  }
  next_separator = ''
  for (let i = 0; i < contexts.length; ++i) {
    const context = contexts[i]
    value = g(context)
    if (i > 0) {
      if (i == contexts.length-1) {
        next_separator = last_separator
      } else {
        next_separator = separator
      }
    }
    s += next_separator + value
  }
  return s
}

class Generator {
  constructor(matcher, action) {
    this.matcher = matcher;
    this.action = action;
  }

  toString() {
    return `Generator(${this.matcher}, ${this.action})`
  }

  matches(objects, context) {
    try {
      return this.matcher({objects, context})
    } catch ( e ) { 
      // don't care
    }
  }

  apply(objects, g, context) {
    return this.action({objects, g, context, gs: gs(g)})
  }
}

class Generators {
  constructor(generators) {
    this.generators = generators
    this.logs = []
  };

  apply(objects, contexts) {
    if (Array.isArray(contexts)) {
    } else if ( typeof contexts == 'object' ) {
      contexts = [contexts]
    } else {
      return String(contexts)
    }
    
    const contexts_prime = [];
    for (const icontext in contexts) {
      const context = contexts[icontext]
      let context_prime = JSON.stringify(context);
      for (let igenerator = 0; igenerator < this.generators.length; ++igenerator) {
        const generator = this.generators[igenerator];
        if (generator.matches(objects, context)) {
          const g = (context) => this.apply(objects, context)
          this.logs.push(`Generators: applied ${generator.toString()}\n to\n ${JSON.stringify(context)}`)
          context_prime = generator.apply(objects, g, context);
          break
        }
      }
      contexts_prime.push(context_prime);
    }
    return contexts_prime;
  }
}

module.exports = { Generator, Generators };
